import os
from typing import Any, Dict

from jinja2 import Environment, FileSystemLoader, StrictUndefined

from .paths import path_head, path_tail


def parse_template(
    template_path: str,
    output_path: str,
    context: Dict[str, Any],
) -> None:
    env = Environment(
        loader=FileSystemLoader(
            path_tail(template_path), encoding="utf-8"
        ),
        trim_blocks=True,
        lstrip_blocks=True,
        keep_trailing_newline=True,
        undefined=StrictUndefined,
    )

    env.filters["path_join"] = lambda paths: os.path.join(*paths)

    with open(output_path, "w") as stream:
        stream.write(
            env.get_template(path_head(template_path)).render(
                **context
            )
        )



import os
from dataclasses import dataclass
from typing import Dict

from quickdocs.utils.paths import path_base


@dataclass
class Paths:
    template_path: str
    template_dir: str
    output_dir: str

    @property
    def dct(self) -> Dict:
        return self.__dict__

    @property
    def local_path(self) -> str:
        return os.path.relpath(self.template_path, self.template_dir)

    @property
    def output_path(self) -> str:
        return os.path.join(self.output_dir, self.local_path)

    @property
    def first_subdir(self):
        return path_base(self.local_path)



import os
import shutil
from argparse import ArgumentParser

from walkmate import get_child_files

from .state.inputs import Inputs
from .state.paths import Paths
from .utils.dicts import merge_dicts
from .utils.jinja import parse_template
from .utils.paths import create_parents, path_head, replace_ext

templates_dir = os.path.normpath(
    os.path.join(__file__, "..", "templates")
)

cli = ArgumentParser("Quickdocs")
cli.add_argument("input", help="Input file path")
cli.add_argument(
    "--output-dir",
    help="Output directory",
    default=os.path.join(os.getcwd(), "docs"),
)


def main() -> None:
    args = cli.parse_args()
    inputs = Inputs.from_file(args.input)

    for template_path in get_child_files(templates_dir):
        paths = Paths(template_path, templates_dir, args.output_dir)
        render_context = merge_dicts(inputs.dct, paths.dct)
        create_parents(paths.output_path)

        if paths.first_subdir == "static":
            shutil.copy(paths.template_path, paths.output_path)

        elif path_head(template_path) == "conf.j2":
            parse_template(
                paths.template_path,
                replace_ext(paths.output_path, ".py"),
                render_context,
            )

        else:
            parse_template(
                paths.template_path,
                replace_ext(paths.output_path, ".rst"),
                render_context,
            )


if __name__ == "__main__":
    main()



import os
from pathlib import Path


def path_head(path: str) -> str:
    return os.path.split(path)[1]


def path_tail(path: str) -> str:
    return os.path.split(path)[0]


def path_base(path: str) -> str:
    return os.path.normpath(path).split(os.sep)[0]


def path_ext(path: str) -> str:
    return os.path.splitext(path)[1]


def replace_ext(path: str, ext: str) -> str:
    return path.replace(os.path.splitext(path)[1], ext)


def create_parents(path: str) -> None:
    Path(path_tail(path)).mkdir(parents=True, exist_ok=True)



from __future__ import annotations

import os
from dataclasses import dataclass
from typing import Dict, Optional, Set, get_type_hints

from quickdocs.exceptions.files import UnrecognisedFormat
from quickdocs.exceptions.inputs import (
    MissingInputs,
    warn_redundant_inputs,
)
from quickdocs.utils.files import parse_json, parse_yaml
from quickdocs.utils.paths import path_ext
from quickdocs.utils.types import cls_defaults


@dataclass
class Inputs:
    project: str
    version: str
    author: str
    html_title: str
    github_url: str

    # Defaults for templating
    debug: bool = False
    project_root: str = os.getcwd()
    verbose_name: Optional[str] = None

    # Readme settings
    markup_readme: bool = True
    readme_name: str = "README.md"

    # Apicdoc settings
    apidoc_module_dir: Optional[str] = None

    @classmethod
    def from_file(cls, path: str) -> Inputs:
        file_type = path_ext(path)

        if file_type == ".json":
            inputs = parse_json(path)

        elif file_type in [".yml", ".yaml"]:
            inputs = parse_yaml(path)

        else:
            raise UnrecognisedFormat(file_type)

        missing_inputs = cls.missing_inputs(set(inputs.keys()))
        if missing_inputs:
            raise MissingInputs(list(missing_inputs))

        redundant_inputs = cls.redundant_inputs(set(inputs.keys()))
        if redundant_inputs:
            warn_redundant_inputs(list(redundant_inputs))

        return cls(**inputs)

    @property
    def dct(self) -> Dict:
        return self.__dict__

    @classmethod
    def allowed_inputs(cls) -> Set[str]:
        return set(get_type_hints(cls).keys())

    @classmethod
    def missing_inputs(cls, inputs: Set[str]) -> Set[str]:
        return cls.allowed_inputs() - cls_defaults(cls) - inputs

    @classmethod
    def redundant_inputs(cls, inputs: Set[str]) -> Set[str]:
        return inputs - cls.allowed_inputs()



import warnings
from typing import List


class MissingInputs(Exception):
    def __init__(self, inputs: List[str]) -> None:
        super().__init__(f"Missing inputs: {inputs}")


def warn_redundant_inputs(inputs: List[str]) -> None:
    warnings.warn(f"Redundant inputs: {inputs}")



from inspect import Parameter, signature
from typing import Any, Set


def cls_defaults(cls: Any) -> Set[str]:
    return set(
        k
        for k, v in signature(cls.__init__).parameters.items()
        if v.default is not Parameter.empty
    )



import json
from typing import Dict

import yaml


def parse_json(path: str) -> Dict:
    with open(path, "r") as stream:
        return json.loads(stream.read())


def parse_yaml(path: str) -> Dict:
    with open(path, "r") as stream:
        return yaml.safe_load(stream)



from setuptools import setup

if __name__ == "__main__":
    setup(
        entry_points={
            "console_scripts": ["quickdocs = quickdocs.__main__:main"]
        },
        install_requires=[
            "pypandoc",
            "pyyaml",
            "walkmate",
            "simple_pipes",
            "sphinx",
            "pyimport",
            "pypandoc",
            "sphinxcontrib.apidoc",
            "sphinxcontrib.pandoc_markdown",
            "sphinx-autodoc-annotation",
            "yummy_sphinx_theme",
        ],
        extras_require={
            "dist": ["wheel", "twine", "bump2version"],
            "tests": [
                "tox",
                "pytest",
                "pytest-cov",
                "pytest-html",
                "pytest-sugar",
                "pytest-bdd",
                "pytest-watch",
            ],
        },
    )
